#include "pch.h"

#include "Contoso_h.h"
#include <wrl.h>
#include <string>
#include <memory>
#include "Calculator.h"

using namespace std;
using namespace Microsoft::WRL;
using namespace Windows::Foundation;

namespace ABI {
    namespace Contoso	{
        class Calculator : public RuntimeClass<ICalculator> {
            InspectableClass(RuntimeClass_Contoso_Calculator, BaseTrust)

        public:

            // Use an external C++ Windows Store App DLL to handle strings
            // note: WRL doesn't permit overloading functions
            HRESULT __stdcall AddStr(_In_ HSTRING a, _In_ HSTRING b, _Out_ int* value)
            {
                // Convert HSTRING values into const wchar_t* so we can
                // pass them into C++ DLL
                const wchar_t* buffera = WindowsGetStringRawBuffer(a, nullptr);
                const wchar_t* bufferb = WindowsGetStringRawBuffer(b, nullptr);

                // Instantiate calculator using modern methods - reference: 
                // http://msdn.microsoft.com/en-us/library/hh279669.aspx
                auto calc = make_shared<WsaDllCpp::Calculator>();

                // Add the string values
                auto val = calc->Add(buffera, bufferb);

                // Assign value
                *value = val;
                return S_OK;
            }

            // http://msdn.microsoft.com/en-us/library/jj155856.aspx
            // Walkthrough: Creating a Basic Windows Runtime Component Using WRL
            HRESULT __stdcall Add(_In_ int a, _In_ int b, _Out_ int* value)	{
                if (value == nullptr)
                {
                    return E_POINTER;
                }
                *value = a + b;
                return S_OK;
            }
        };
        ActivatableClass(Calculator);
    }
}