﻿// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (c) Microsoft Corporation. All rights reserved
#include "pch.h"
#include "DelegateCommand.h"
#include <assert.h>
using namespace Platform;
using namespace WsaWrcCpp::DataBinding;

/// <summary>
/// Initializes a new instance of the <see cref="DelegateCommand"/> class.
/// </summary>
DelegateCommand::DelegateCommand() {}

/// <summary>
/// Initializes a new instance of the <see cref="DelegateCommand"/> class.
/// </summary>
/// <param name="execute">The execute.</param>
/// <param name="canExecute">The can execute.</param>
DelegateCommand::DelegateCommand(ExecuteDelegate^ execute, CanExecuteDelegate^ canExecute) 
	: m_executeDelegate(execute), 
	  m_canExecuteDelegate(canExecute){}

/// <summary>
/// Executes the specified parameter.
/// </summary>
/// <param name="parameter">The parameter.</param>
void DelegateCommand::Execute(Object^ parameter) {
    assert(m_executeDelegate != nullptr);
    if (nullptr != m_executeDelegate)  {
        m_executeDelegate(parameter);
    }
}

/// <summary>
/// Determines whether this instance can execute the specified parameter.
/// </summary>
/// <param name="parameter">The parameter.</param>
bool DelegateCommand::CanExecute(Object^ parameter) {
    if (m_canExecuteDelegate == nullptr)
    {
        return true;
    }

    bool m_canExecute = m_canExecuteDelegate(parameter);
    
    // BillKrat.2014.01.11 - Added RaiseCanExecuteChanged
    // bool temp = m_canExecuteDelegate(parameter);
    // if (m_canExecute != temp)
    // {
    //     m_canExecute = temp;
    //     CanExecuteChanged(this, nullptr);
    // }

    return m_canExecute;
}

/// <summary>
/// Raises the can execute changed event.
/// </summary>
void DelegateCommand::RaiseCanExecuteChanged() {
	CanExecuteChanged(nullptr, nullptr);
}