#include "pch.h"
#include "MainViewModel.h"
#include "DelegateCommand.h"
#include "MockDataDal.h"

using namespace Platform;
using namespace WsaWrcCpp::ViewModel;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Input;
using namespace WsaWrcCpp::DataLayer;

// Helper template - RaiseCanExecuteChanged is a method on 
// DelegateCommand and is not available on ICommand.  We'll cast 
// instance to T and then call RaiseCanExecuteChanged which will 
// force each Command::CanExecute() to be called
template <typename T>
static void RaiseCanExecuteChanged(ICommand^ instance) 
{	
	auto command = dynamic_cast<T^>(instance);
	command->RaiseCanExecuteChanged();
}


MainViewModel::MainViewModel(void)
{

}

// called by View code-behind.
void MainViewModel::Initialize()
{
	// Wire up button click events
	ButtonCommand = ref new DelegateCommand(
		ref new ExecuteDelegate(this, &MainViewModel::Execute),
		ref new CanExecuteDelegate(this, &MainViewModel::CanExecute));
	
	NotifyPropertyChanged("ButtonCommand");

	// In the real world we would instantiate a BLL which will 
	// access the DAL
	auto dal = ref new MockDataDal();

	// Set list and selected (notify events will fire)
	NameValueList = dal->GetDefaultList();
	SelectedNameValue = NameValueList->GetAt(0);

}
void MainViewModel::Execute(Object^ parameter)
{
	// Cast parameter to button
	auto button = dynamic_cast<Button^>(parameter);

	if(button->Name=="btnReset")
	{
		CheckBox1 = false;
		CheckBox2 = false;
		StatusResults = "Checkboxes reset to false";
	} 

	if(button->Name=="btnClickMe")
	{
		auto comboName = SelectedNameValue->Name;
		auto comboValue = SelectedNameValue->Value;

		StatusResults = "CheckBox1="+ CheckBox1.ToString() 
			          + "\t\tCheckBox2="+ CheckBox2.ToString()
					  + "\r\nCombo Name="+comboName
					  + "\t\tValue="+comboValue;
	}

	if (Presenter != nullptr)
		Presenter->CommandHandler(parameter);

}
bool MainViewModel::CanExecute(Object^ parameter)
{
	auto button = dynamic_cast<Button^>(parameter);

	// ClickMe button is always enabled
	if(button->Name=="btnClickMe")
		return true;

	// Only enable if one of the checkboxes is checked
	return CheckBox1 || CheckBox2; 
}


IVector<NameValuePair^>^ MainViewModel::NameValueList::get() { return _NameValueList; }
void MainViewModel::NameValueList::set(IVector<NameValuePair^>^ value)
{
	_NameValueList = value;
	NotifyPropertyChanged("NameValueList");
}

NameValuePair^ MainViewModel::SelectedNameValue::get() { return _SelectedNameValue; }
void MainViewModel::SelectedNameValue::set(NameValuePair^ value) 
{
	_SelectedNameValue = value;
	NotifyPropertyChanged("SelectedNameValue");
}



/// <summary>
/// PROPERTY: CheckBox1 
/// </summary>
bool MainViewModel::CheckBox1::get(){return _checkBox1;}
void MainViewModel::CheckBox1::set(bool value)
{
	_checkBox1 = value;
	NotifyPropertyChanged("CheckBox1");
	RaiseCanExecuteChanged<DelegateCommand>(ButtonCommand);
}

/// <summary>
/// PROPERTY: CheckBox2
/// </summary>
bool MainViewModel::CheckBox2::get(){return _checkBox2;}
void MainViewModel::CheckBox2::set(bool value)
{
	_checkBox2 = value;
	NotifyPropertyChanged("CheckBox2");
	RaiseCanExecuteChanged<DelegateCommand>(ButtonCommand);
}

/// <summary>
/// PROPERTY: StatusResults
/// </summary>
String^ MainViewModel::StatusResults::get(){return _statusResults;}
void MainViewModel::StatusResults::set(String^ value)
{
	_statusResults = value;
	NotifyPropertyChanged("StatusResults");
}







