#include "pch.h"
#include "DataBindingPresenter.h"
#include "DataBindingView.xaml.h"

using namespace WsaXamlCpp;
using namespace WsaWrcCpp::ViewModel;
using namespace WsaWrcCpp::DataLayer;


/// <summary>
/// Called by NavigationHelper::NavigatedTo() method after LoadState
/// </summary>
/// <param name="view">reference to instantiated view</param>
void DataBindingPresenter::Initialize(Object^ view)
{
	_view = safe_cast<DataBindingView^>(view);
	
	_logger->Log("Instantiate view model");
	_viewModel = ref new MainViewModel();

	_logger->Log("Attach to the View");
	_view->DataContext = _viewModel;

	_logger->Log("Initialize view model");
	_viewModel->Initialize();

	_logger->Log("Setting list values on view model");

	// In the real world a business logic layer would be called to 
	// retrieve data. The BLL would access the DAL and return list
	auto dal = ref new MockDataDal();
	for (auto const item : dal->GetCowboyList())
		_viewModel->NameValueList->Append(item);

	// Default to first record
	_viewModel->SelectedNameValue = _viewModel->NameValueList->GetAt(0);
}

void DataBindingPresenter::Navigate(Frame^ frame)
{
	_logger->Log("Navigating to view", "NAVIGATION", "5");
	frame->Navigate(TypeName(WsaXamlCpp::DataBindingView::typeid), this);
}

void DataBindingPresenter::CommandHandler(Object^ command)
{
	_logger->Log("CommandHandler executed", "COMMAND", "1");
	// We'll keep default behavior...
}

DataBindingPresenter::DataBindingPresenter()
{
	_logger = ref new DebugLogger();
}

