#include "pch.h"
#include "ReusedPresenter.h"
#include "DataBindingPresenter.h"
#include "DataBindingView.xaml.h"

using namespace WsaXamlCpp;
using namespace WsaWrcCpp::ViewModel;
using namespace WsaWrcCpp::DataLayer;

/// <summary>
/// Called by NavigationHelper::NavigatedTo() method after LoadState
/// </summary>
/// <param name="view">reference to instantiated view</param>
void ReusedPresenter::Initialize(Object^ view)
{
	_view = safe_cast<DataBindingView^>(view);

	// Instantiate view model
	_viewModel = ref new MainViewModel();

	// Attach to the View
	_view->DataContext = _viewModel;

	// Find the title on the view so we can replace the text
	auto title = dynamic_cast<TextBlock^>(_view->FindName("pageTitle"));
	title->Text = "Reused View and ViewModel";

	// Initialize view model
	_viewModel->Initialize();

	// Give reference to presenter because we are overriding command handler
	_viewModel->Presenter = (IPresenter^)this;

	// We want only the hiding user list
	_viewModel->NameValueList->Clear();

	// In the real world a business logic layer would be called to retrieve data.
	// The BLL would access the DAL and return list
	auto dal = ref new MockDataDal();
	for (auto item : dal->GetHidingList())
		_viewModel->NameValueList->InsertAt(0, item);

	// Default to John Smith
	_viewModel->SelectedNameValue = _viewModel->NameValueList->GetAt(1);
}

void ReusedPresenter::Navigate(Frame^ frame)
{
	frame->Navigate(TypeName(WsaXamlCpp::DataBindingView::typeid), this);
}

void ReusedPresenter::CommandHandler(Object^ command)
{
	// Presenter knows only buttons are being handled by view model
	auto button = dynamic_cast<Button^>(command);

	// Do something different - we could totally rewrite StatusResult but
	// we'll simply append for demonstration purposes
	_counter++;
	_viewModel->StatusResults += 
		"\r\n\r\nOverride Status Results " + _counter + 
		"\r\nButton clicked =>" + button->Name;

}

ReusedPresenter::ReusedPresenter()
{
}
