﻿Imports System.Linq
Imports Windows.Foundation.Collections
Imports System.Collections.Generic

Namespace WsaXamlCs.Common
    ''' <summary>
    ''' Implementation of IObservableMap that supports reentrancy for use as a default view
    ''' model.
    ''' </summary>
    Public Class ObservableDictionary
        Implements IObservableMap(Of String, Object)
        Private Class ObservableDictionaryChangedEventArgs
            Implements IMapChangedEventArgs(Of String)
            Public Sub New(ByVal change As CollectionChange, ByVal key As String)
                Me.privateCollectionChange = change
                Me.privateKey = key
            End Sub

            Private privateCollectionChange As CollectionChange

            Public ReadOnly Property CollectionChange() As CollectionChange Implements IMapChangedEventArgs(Of String).CollectionChange
                Get
                    Return privateCollectionChange
                End Get
            End Property
            Private privateKey As String
            Public ReadOnly Property Key() As String Implements IMapChangedEventArgs(Of String).Key
                Get
                    Return privateKey
                End Get
            End Property
        End Class

        Private _dictionary As New Dictionary(Of String, Object)()
        Public Event MapChanged As MapChangedEventHandler(Of String, Object) Implements IObservableMap(Of String, Object).MapChanged

        Private Sub InvokeMapChanged(ByVal change As CollectionChange, ByVal key As String)
            RaiseEvent MapChanged(Me, New ObservableDictionaryChangedEventArgs(change, key))
        End Sub

        Public Sub Add(ByVal key As String, ByVal value As Object) Implements IDictionary(Of String, Object).Add
            Me._dictionary.Add(key, value)
            Me.InvokeMapChanged(CollectionChange.ItemInserted, key)
        End Sub

        Public Sub Add(ByVal item As KeyValuePair(Of String, Object)) Implements ICollection(Of KeyValuePair(Of String, Object)).Add
            Me.Add(item.Key, item.Value)
        End Sub

        Public Function Remove(ByVal key As String) As Boolean Implements IDictionary(Of String, Object).Remove
            If Me._dictionary.Remove(key) Then
                Me.InvokeMapChanged(CollectionChange.ItemRemoved, key)
                Return True
            End If
            Return False
        End Function

        Public Function Remove(ByVal item As KeyValuePair(Of String, Object)) As Boolean Implements ICollection(Of KeyValuePair(Of String, Object)).Remove
            Dim currentValue As Object = String.Empty
            If Me._dictionary.TryGetValue(item.Key, currentValue) AndAlso Object.Equals(item.Value, currentValue) AndAlso Me._dictionary.Remove(item.Key) Then
                Me.InvokeMapChanged(CollectionChange.ItemRemoved, item.Key)
                Return True
            End If
            Return False
        End Function

        Default Property Item(ByVal key As String) As Object Implements IDictionary(Of String, Object).Item
            Get
                Return Me._dictionary(key)
            End Get
            Set(ByVal value As Object)
                Me._dictionary(key) = value
                Me.InvokeMapChanged(CollectionChange.ItemChanged, key)
            End Set
        End Property

        Public Sub Clear() Implements ICollection(Of KeyValuePair(Of String, Object)).Clear
            Dim priorKeys As IEnumerable(Of String) = Me._dictionary.Keys.ToArray()
            Me._dictionary.Clear()
            For Each key As String In priorKeys
                Me.InvokeMapChanged(CollectionChange.ItemRemoved, key)
            Next key
        End Sub

        Public ReadOnly Property Keys() As ICollection(Of String) Implements IDictionary(Of String, Object).Keys
            Get
                Return Me._dictionary.Keys
            End Get
        End Property

        Public Function ContainsKey(ByVal key As String) As Boolean Implements IDictionary(Of String, Object).ContainsKey
            Return Me._dictionary.ContainsKey(key)
        End Function

        Public Function TryGetValue(ByVal key As String, <System.Runtime.InteropServices.Out()> ByRef value As Object) As Boolean Implements IDictionary(Of String, Object).TryGetValue
            Return Me._dictionary.TryGetValue(key, value)
        End Function

        Public ReadOnly Property Values() As ICollection(Of Object) Implements IDictionary(Of String, Object).Values
            Get
                Return Me._dictionary.Values
            End Get
        End Property

        Public Function Contains(ByVal item As KeyValuePair(Of String, Object)) As Boolean Implements ICollection(Of KeyValuePair(Of String, Object)).Contains
            Return Me._dictionary.Contains(item)
        End Function

        Public ReadOnly Property Count() As Integer Implements ICollection(Of KeyValuePair(Of String, Object)).Count
            Get
                Return Me._dictionary.Count
            End Get
        End Property

        Public ReadOnly Property IsReadOnly() As Boolean Implements ICollection(Of KeyValuePair(Of String, Object)).IsReadOnly
            Get
                Return False
            End Get
        End Property

        Public Function GetEnumerator() As IEnumerator(Of KeyValuePair(Of String, Object)) Implements IEnumerable(Of KeyValuePair(Of String, Object)).GetEnumerator
            Return Me._dictionary.GetEnumerator()
        End Function

        Private Function IEnumerable_GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
            Return Me._dictionary.GetEnumerator()
        End Function

        Public Sub CopyTo(ByVal array() As KeyValuePair(Of String, Object), ByVal arrayIndex As Integer) Implements ICollection(Of KeyValuePair(Of String, Object)).CopyTo
            Dim arraySize As Integer = array.Length
            For Each pair As KeyValuePair(Of String, Object) In Me._dictionary
                If arrayIndex >= arraySize Then
                    Exit For
                End If
                array(arrayIndex) = pair
                arrayIndex += 1
            Next pair
        End Sub
    End Class
End Namespace
